# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


class EntityInterface(metaclass=ABCMeta):
    """
    Capabilities shared between scene objects.
    """

    @abstractmethod
    def __init__(
        self, name: "str", description: "str| None" = None, native_instance=None
    ):

        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def is_exists(
        self,
    ) -> "bool":
        """
        Check if an entity exists
        """

        print("is_exists is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def rename(
        self, new_name: "str", renamelinked_entities_and_landmarks: "bool" = True
    ) -> Self:
        """
        Rename the entity, with an option to rename linked landmarks and underlying data.
        """

        print("rename is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def delete(self, remove_children: "bool" = True) -> Self:
        """
        Delete the entity from the scene. You may need to delete an associated joint or other features.
        """

        print("delete is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def is_visible(
        self,
    ) -> "bool":
        """
        Returns whether the entity is visible in the scene.
        """

        print(
            "is_visible is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_visible(self, is_visible: "bool") -> Self:
        """
        Toggles visibility of an entity in the scene.
        """

        print(
            "set_visible is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def apply(
        self,
        rotation: "bool" = True,
        scale: "bool" = True,
        location: "bool" = False,
        modifiers: "bool" = True,
    ) -> Self:
        """
        Apply any modifications. This is application specific, but a general function is that it finalizes any changes made to an entity.
        """

        print("apply is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def get_native_instance(
        self,
    ) -> "object":
        """
        Get the native API's object instance. For example, in Blender API, this would return a bpy.object instance.
        """

        print(
            "get_native_instance is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_location_world(
        self,
    ) -> "Point":
        """
        Get the entities XYZ location relative to World Space.
        """

        print(
            "get_location_world is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_location_local(
        self,
    ) -> "Point":
        """
        Get the entities XYZ location relative to Local Space.
        """

        print(
            "get_location_local is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def select(
        self,
    ) -> Self:
        """
        Select the entity (in UI).
        """

        print("select is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def translate_xyz(
        self,
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> Self:
        """
        Translate in the XYZ directions. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print(
            "translate_xyz is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def translate_x(self, amount: "str|float|Dimension") -> Self:
        """
        Translate in the X direction. Pass a number or Dimension or Dimension-String (e.g. '2cm') to translate to a specific length.
        """

        print(
            "translate_x is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def translate_y(self, amount: "str|float|Dimension") -> Self:
        """
        Translate in the Y direction. Pass a number or Dimension or Dimension-String (e.g. '2cm') to translate to a specific length.
        """

        print(
            "translate_y is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def translate_z(self, amount: "str|float|Dimension") -> Self:
        """
        Translate in the z direction. Pass a number or Dimension or Dimension-String (e.g. '2cm') to translate to a specific length.
        """

        print(
            "translate_z is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def rotate_xyz(
        self, x: "str|float|Angle", y: "str|float|Angle", z: "str|float|Angle"
    ) -> Self:
        """
        Rotate in the XYZ direction. Default units is degrees. Pass in a number, Angle or Angle-String (e.g. 'PI/4radians' or 'PI/4r' or '90d'
        """

        print(
            "rotate_xyz is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def rotate_x(self, rotation: "str|float|Angle") -> Self:
        """
        Rotate in the X direction. Default units is degrees. Pass in a number, Angle or Angle-String (e.g. 'PI/4radians' or 'PI/4r' or '90d'
        """

        print("rotate_x is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def rotate_y(self, rotation: "str|float|Angle") -> Self:
        """
        Rotate in the Y direction. Default units is degrees. Pass in a number, Angle or Angle-String (e.g. 'PI/4radians' or 'PI/4r' or '90d'
        """

        print("rotate_y is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def rotate_z(self, rotation: "str|float|Angle") -> Self:
        """
        Rotate in the Z direction. Default units is degrees. Pass in a number, Angle or Angle-String (e.g. 'PI/4radians' or 'PI/4r' or '90d'
        """

        print("rotate_z is called in an abstract method. Please override this method.")

        raise NotImplementedError()

    @abstractmethod
    def get_bounding_box(
        self,
    ) -> "BoundaryBox":
        """
        Get the Boundary Box around the entity.
        """

        print(
            "get_bounding_box is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def get_dimensions(
        self,
    ) -> "Dimensions":
        """
        Get the bounding-box dimensions in each axis (X,Y,Z).
        """

        print(
            "get_dimensions is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.edge_interface import EdgeInterface


from codetocad.interfaces.vertex_interface import VertexInterface

from codetocad.interfaces.landmark_interface import LandmarkInterface


from codetocad.interfaces.projectable_interface import ProjectableInterface


from codetocad.interfaces.entity_interface import EntityInterface


from codetocad.proxy.vertex import Vertex

from codetocad.proxy.landmark import Landmark


from providers.sample.entity import Entity


class Edge(EdgeInterface, Entity):

    def __init__(
        self,
        name: "str",
        v1: "VertexInterface",
        v2: "VertexInterface",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.v1 = v1
        self.v2 = v2
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    def offset(self, distance: "str|float|Dimension") -> "EdgeInterface":

        print("offset called", f": {distance}")

        return Edge(
            v1=Vertex("a vertex", Point.from_list_of_float_or_string([0, 0, 0])),
            v2=Vertex("a vertex", Point.from_list_of_float_or_string([0, 0, 0])),
            name="an edge",
        )

    def fillet(self, other_edge: "EdgeInterface", amount: "str|float|Angle") -> Self:

        print("fillet called", f": {other_edge}, {amount}")

        return self

    def set_is_construction(self, is_construction: "bool") -> Self:

        print("set_is_construction called", f": {is_construction}")

        return self

    def get_is_construction(
        self,
    ) -> "bool":

        print(
            "get_is_construction called",
        )

        return True

    def mirror(
        self,
        mirror_across_entity: "str|EntityInterface",
        axis: "str|int|Axis",
        resulting_mirrored_entity_name: "str| None" = None,
    ) -> Self:

        print(
            "mirror called",
            f": {mirror_across_entity}, {axis}, {resulting_mirrored_entity_name}",
        )

        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: "str|float|Dimension",
        direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "linear_pattern called", f": {instance_count}, {offset}, {direction_axis}"
        )

        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: "str|float|Angle",
        center_entity_or_landmark: "str|EntityInterface",
        normal_direction_axis: "str|int|Axis" = "z",
    ) -> Self:

        print(
            "circular_pattern called",
            f": {instance_count}, {separation_angle}, {center_entity_or_landmark}, {normal_direction_axis}",
        )

        return self

    def remesh(self, strategy: "str", amount: "float") -> Self:

        print("remesh called", f": {strategy}, {amount}")

        return self

    def subdivide(self, amount: "float") -> Self:

        print("subdivide called", f": {amount}")

        return self

    def decimate(self, amount: "float") -> Self:

        print("decimate called", f": {amount}")

        return self

    def project(self, project_from: "ProjectableInterface") -> "ProjectableInterface":

        print("project called", f": {project_from}")

        return __import__("codetocad").Sketch("a projected sketch")

    def create_landmark(
        self,
        landmark_name: "str",
        x: "str|float|Dimension",
        y: "str|float|Dimension",
        z: "str|float|Dimension",
    ) -> "LandmarkInterface":

        print("create_landmark called", f": {landmark_name}, {x}, {y}, {z}")

        return Landmark("name", "parent")

    def get_landmark(self, landmark_name: "str|PresetLandmark") -> "LandmarkInterface":

        print("get_landmark called", f": {landmark_name}")

        return Landmark("name", "parent")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


class ProjectableInterface(metaclass=ABCMeta):
    """
    This entity can be projected onto a surface or accept a projection
    """

    @abstractmethod
    def project(self, project_from: "ProjectableInterface") -> "ProjectableInterface":
        """
        Project another entity onto this one.
        """

        print("project is called in an abstract method. Please override this method.")

        raise NotImplementedError()



import math
from codetocad.core.dimension import Dimension
from codetocad.core.math_utils import linspace
from codetocad.core.point import Point


def get_point_on_circle_at_angle(theta_radians: float, radius: Dimension):
    return Point(
        x=radius * math.cos(theta_radians),
        y=radius * math.sin(theta_radians),
        z=Dimension.zero(radius.unit),
    )


def get_circle_points(radius: Dimension, resolution: int):
    """
    Returns points that make up a circle.
    NOTE: The first and last points are the same to denote a closed shape.
    """
    return [
        get_point_on_circle_at_angle(theta, radius)
        for theta in linspace(0.0, math.pi * 2, resolution, endpoint=False)
    ] + [get_point_on_circle_at_angle(0.0, radius)]


def get_center_of_circle(point1: Point, point2: Point, radius: Dimension):
    # references https://rosettacode.org/wiki/Circles_of_given_radius_through_two_points#Python
    (x1, y1), (x2, y2) = (point1.x, point1.y), (point2.x, point2.y)

    dx, dy = x2 - x1, y2 - y1

    cord_length = (dx.raise_power(2) + dy.raise_power(2)).raise_power(1 / 2)

    # halfway point
    x3, y3 = (x1 + x2) / 2, (y1 + y2) / 2

    mirror_line_distance = (
        radius.raise_power(2) - (cord_length / 2).raise_power(2)
    ).raise_power(1 / 2)

    # # The other answer
    # c2 = Cir(x=x3 + d*dy/q,
    #          y=y3 - d*dx/q,
    #          r=abs(r))

    return Point(
        x=x3 - mirror_line_distance * dy / cord_length,
        y=y3 + mirror_line_distance * dx / cord_length,
        z=Dimension.zero(),
    )



from codetocad.launchers.blender import run_blender_process
from codetocad.launchers.launcher_args import LauncherArgs
from codetocad.launchers.sample import run_with_sample
from codetocad.launchers.unknown_launcher import run_unknown_process


def execute_launcher():
    """
    Builds a LauncherArgs instance from cli args, then attempts to run known launchers, and if it fails to do that, runs an unknown process.
    """
    args = LauncherArgs.from_cli_args()

    # Sample/Dummy/Mock launcher:
    if args.is_sample_launcher():
        print("Running script with the sample provider.")
        run_with_sample(args)
        return

    launcher_lower = args.launcher.lower()

    # Known launchers:
    if launcher_lower == "blender":
        return run_blender_process(args)

    # Execute an unknown process:
    run_unknown_process(args)


if __name__ == "__main__":
    execute_launcher()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from codetocad.codetocad_types import *


from codetocad.interfaces.landmark_interface import LandmarkInterface


from codetocad.interfaces.entity_interface import EntityInterface


from providers.sample.entity import Entity


class Landmark(LandmarkInterface, Entity):

    def __init__(
        self,
        name: "str",
        parent_entity: "str|EntityInterface",
        description: "str| None" = None,
        native_instance=None,
    ):

        self.name = name
        self.parent_entity = parent_entity
        self.description = description
        self.native_instance = native_instance

    def clone(
        self,
        new_name: "str",
        offset: "str|list[str]|list[float]|list[Dimension]|Dimensions| None" = None,
        new_parent: "str|EntityInterface| None" = None,
    ) -> "LandmarkInterface":

        print("clone called", f": {new_name}, {offset}, {new_parent}")

        return Landmark("name", "parent")

    def get_landmark_entity_name(
        self,
    ) -> "str":

        print(
            "get_landmark_entity_name called",
        )

        return "String"

    def get_parent_entity(
        self,
    ) -> "EntityInterface":

        print(
            "get_parent_entity called",
        )

        return __import__("codetocad").Part("an entity")



import runpy
from codetocad.launchers.launcher_args import LauncherArgs


def run_with_sample(launcher_args: LauncherArgs):
    """
    Launches the script with the providers.sample module.
    """
    return runpy.run_path(launcher_args.script_file_path, run_name="__main__")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from abc import ABCMeta, abstractmethod


from codetocad.codetocad_types import *

from typing import Self


from codetocad.interfaces.projectable_interface import ProjectableInterface

from codetocad.interfaces.entity_interface import EntityInterface


class VertexInterface(EntityInterface, ProjectableInterface, metaclass=ABCMeta):
    """
    A single point in space, or a control point.
    """

    @abstractmethod
    def __init__(
        self,
        name: "str",
        location: "Point",
        description: "str| None" = None,
        native_instance=None,
        parent_entity: "str|EntityInterface| None" = None,
    ):

        self.name = name
        self.location = location
        self.description = description
        self.native_instance = native_instance
        self.parent_entity = parent_entity

    @abstractmethod
    def get_control_points(
        self,
    ) -> "list[Point]":
        """
        Get a vertex's curve control points. This may not be applicable for some curve types.
        """

        print(
            "get_control_points is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()

    @abstractmethod
    def set_control_points(
        self, points: "list[str|list[str]|list[float]|list[Dimension]|Point]"
    ) -> Self:
        """
        Set a vertex's curve control points. This may not be applicable for some curve types.
        """

        print(
            "set_control_points is called in an abstract method. Please override this method."
        )

        raise NotImplementedError()



import unittest

from providers.blender.blender_fake.mock_modeling_provider import (
    reset_mock_modeling_provider,
    inject_mock_modeling_provider,
)


class TestBlenderActions(unittest.TestCase):
    def setUp(self) -> None:
        # NOTE: if you're running this test locally, you might need `pip install fake-bpy-module-latest` or Blender API added to python path
        reset_mock_modeling_provider()
        inject_mock_modeling_provider(globals())
        super().setUp()

    def test_todo(self):
        pass
